@@ -44,7 +44,7 @@
 using namespace SooperLooper;
 using namespace std;
 using namespace PBD;
-using namespace SigC;
+using namespace sigc;
 
 #define MAX_EVENTS 1024
 #define MAX_SYNC_EVENTS 1024
@@ -113,6 +113,8 @@ Engine::Engine ()
 	_send_midi_start_on_trigger = false;
 	_send_midi_start_after_next_hit = false;
 
+	_load_sess_event = NULL;
+
 	// for now just use the current time!
 	_unique_id = (int) ::time(NULL);
 
@@ -201,7 +203,7 @@ bool Engine::initialize(AudioDriver * driver, int buschans, int port, string pin
 		return false;
 	}
 
-	_driver->ConnectionsChanged.connect(slot(*this, &Engine::connections_changed));
+	_driver->ConnectionsChanged.connect(mem_fun(*this, &Engine::connections_changed));
 	
 	_ok = true;
 
@@ -304,14 +306,14 @@ void Engine::set_midi_bridge (MidiBridge * bridge)
 {
 	_midi_bridge = bridge;
 	if (_midi_bridge) {
-		_midi_bridge->BindingLearned.connect(slot(*this, &Engine::binding_learned));
-		_midi_bridge->NextMidiReceived.connect(slot(*this, &Engine::next_midi_received));
+		_midi_bridge->BindingLearned.connect(mem_fun(*this, &Engine::binding_learned));
+		_midi_bridge->NextMidiReceived.connect(mem_fun(*this, &Engine::next_midi_received));
 
-		_midi_bridge->MidiCommandEvent.connect (slot(*this, &Engine::push_midi_command_event));
-		_midi_bridge->MidiControlEvent.connect (slot(*this, &Engine::push_midi_control_event));
-		_midi_bridge->MidiSyncEvent.connect (slot(*this, &Engine::push_sync_event));
+		_midi_bridge->MidiCommandEvent.connect (mem_fun(*this, &Engine::push_midi_command_event));
+		_midi_bridge->MidiControlEvent.connect (mem_fun(*this, &Engine::push_midi_control_event));
+		_midi_bridge->MidiSyncEvent.connect (mem_fun(*this, &Engine::push_sync_event));
 
-		ParamChanged.connect(bind (slot(*_midi_bridge, &MidiBridge::parameter_changed), this));
+		ParamChanged.connect(bind (mem_fun(*_midi_bridge, &MidiBridge::parameter_changed), this));
 
 		_midi_bridge->set_output_midi_clock(_output_midi_clock);
 	}
@@ -605,9 +607,7 @@ Engine::remove_loop (Looper * looper)
 	
 	delete looper;
 
-	if (!_loading) {
-		LoopRemoved(); // emit
-	}
+	LoopRemoved(); // emit
 	
 	update_sync_source();
 
@@ -738,6 +738,13 @@ void Engine::process_rt_loop_manage_events ()
 			_rt_instances.push_back (lmevt->looper);
 			lmevt->looper->recompute_latencies();
 		}
+		else if (lmevt->etype == LoopManageEvent::LoadSession)
+		{
+			_rt_instances.clear();
+
+			// signal main loop
+			push_loop_manage_to_main (*lmevt);
+		}
 		
 		_loop_manage_to_rt_queue->increment_read_ptr(1);
 	}
@@ -1420,7 +1427,10 @@ Engine::load_midi_bindings (std::istream & instream, bool append, CommandMap & c
 {
 	char  line[200];
 
-	
+	if (!_midi_bridge) {
+        return false;
+    }
+    
 	if (!append) {
 		_midi_bridge->bindings().clear_bindings();
 	}
@@ -1602,6 +1612,10 @@ Engine::mainloop()
 			if (lmevt->etype == LoopManageEvent::RemoveLoop) {
 				remove_loop (lmevt->looper);
 			}
+			else if (lmevt->etype == LoopManageEvent::LoadSession)
+			{
+				handle_load_session_event();
+			}
 			
 			_loop_manage_to_main_queue->increment_read_ptr(1);
 		}
@@ -1921,7 +1935,9 @@ Engine::process_nonrt_event (EventNonRT * event)
 	{
 		_osc->finish_update_event (*cu_event);
 	}
-	else if ((cl_event = dynamic_cast<ConfigLoopEvent*> (event)) != 0)
+	// to be on the safe side we do not handle loop add and remove commands
+	// while loading a new session
+	else if ((cl_event = dynamic_cast<ConfigLoopEvent*> (event)) != 0 && !_loading)
 	{
 		if (cl_event->type == ConfigLoopEvent::Add) {
 			// todo: use secs
@@ -2054,9 +2070,10 @@ Engine::process_nonrt_event (EventNonRT * event)
 	else if ((sess_event = dynamic_cast<SessionEvent*> (event)) != 0)
 	{
 		if (sess_event->type == SessionEvent::Load) {
-			if (!load_session (sess_event->filename)) {
-				_osc->send_error(sess_event->ret_url, sess_event->ret_path, "Session Load Failed");
-			}
+			_loading = true;
+			_load_sess_event = new SessionEvent(*sess_event);
+			LoopManageEvent lmev (LoopManageEvent::LoadSession, NULL);
+			push_loop_manage_to_rt (lmev);
 		}
 		else {
 			if (!save_session (sess_event->filename, sess_event->write_audio)) {
@@ -2684,6 +2701,25 @@ Engine::generate_sync (nframes_t offset, nframes_t nframes)
 	return hit_at;
 }
 
+void
+Engine::handle_load_session_event ()
+{
+	while (_instances.size() > 0)
+	{
+		remove_loop(_instances.back());
+	}
+	if (!_load_sess_event)
+	{
+		_loading = false;
+		return;
+	}
+	if (!load_session (_load_sess_event->filename))
+	{
+		_osc->send_error(_load_sess_event->ret_url, _load_sess_event->ret_path, "Session Load Failed");
+	}
+	_loading = false;
+	delete _load_sess_event;
+}
 
 bool
 Engine::load_session (std::string fname, string * readstr)
